/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.example.order.manager.web.retail;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.bluestemsoftware.open.eoa.example.order.manager.service.OrderProcessor;
import org.bluestemsoftware.open.eoa.example.order.manager.service.fault.OrderNotFoundFault;
import org.bluestemsoftware.open.eoa.example.order.manager.service.vo.CustomerOrder;
import org.bluestemsoftware.open.eoa.example.order.manager.service.vo.CustomerOrder.Status;
import org.bluestemsoftware.open.eoa.example.order.manager.service.vo.Item;
import org.springframework.validation.BindException;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;
import org.springframework.web.servlet.view.RedirectView;
import org.w3c.dom.Element;

public class StatusController extends SimpleFormController {

    private OrderProcessor orderProcessor;

    @Override
    protected Object formBackingObject(HttpServletRequest request) throws Exception {

        // create an empty order which will be bound to form
        // fields by spring with user entered data on submit.

        Set<Item> items = new HashSet<Item>();
        CustomerOrder co = new CustomerOrder();
        co.setItems(items);
        return co;

    }

    /*
     * (non-Javadoc)
     * @see org.springframework.web.servlet.mvc.SimpleFormController#onSubmit(javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse, java.lang.Object,
     *      org.springframework.validation.BindException)
     */
    @SuppressWarnings("unchecked")
    protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {

        if (request.getParameter("cancel") != null) {
            return new ModelAndView(new RedirectView("home.htm"));
        }
        
        CustomerOrder customerOrder = (CustomerOrder)command;
        Status status = null;
        try {
            status = orderProcessor.getOrderStatus(customerOrder.getOrderNumber());
            customerOrder.setStatus(status);
        } catch (OrderNotFoundFault fault) {
            Element payload = fault.getPayload();
            String orderNumber = payload.getAttribute("orderNumber");
            String message = "Order '" + orderNumber + "' was not found on our system.";
            errors.reject("", message);
            return showForm(request, response, errors);
        }

        Map<String, Object> model = errors.getModel();
        model.put("result", customerOrder);
        return new ModelAndView("inform", model);

    }

    /*
     * Spring dependency injector.
     */
    public void setOrderProcessor(OrderProcessor orderProcessor) {
        this.orderProcessor = orderProcessor;
    }

}
